package org.picocontainer.tck;

import junit.framework.TestCase;

import java.util.ArrayList;
import java.util.List;

/**
 * This class tests objects for conformance with the equals() contract
 * http://java.sun.com/j2se/1.4.2/docs/api/java/lang/Object.html#equals(java.lang.Object)
 *
 * @author Aslak Helles&oslash;y
 * @version $Revision$
 */
public class EqualsTestCase extends TestCase {
    public static interface Tester {
        void compare(Object o1, Object o2);
    }

    private List objects = new ArrayList();

    protected void setUp() throws Exception {
        objects.add("hello");
        objects.add("world");
        objects.add("blah");
    }

    public void testSameObjectsShouldBeReflexive() {
        testAll(objects.toArray(), new Tester(){
            public void compare(Object o1, Object o2) {
                if(o1 == o2) {
                    assertTrue("Should be reflexive", o1.equals(o2));
                }
            }
        });
    }

    public void testObjectsShouldBeSymetricallyEqual() {
        testAll(objects.toArray(), new Tester(){
            public void compare(Object o1, Object o2) {
                if(o1.equals(o2)) {
                    assertTrue("Should be symmetric", o2.equals(o1));
                }
            }
        });
    }

    public void testEqualityShouldBeTransitive() {
        testAll(objects.toArray(), new Tester(){
            public void compare(Object o1, Object o2) {
                if(o1.equals(o2)) {
                    assertTrue("Should be symmetric", o2.equals(o1));
                }
            }
        });
    }

    public void testAll(Object[] oa, Tester tester) {
        for (int i = 0; i < oa.length; i++) {
            Object o1 = oa[i];
            for (int j = oa.length - 1; j >= 0; j--) {
                Object o2 = oa[j];
                tester.compare(o1, o2);
            }
        }
    }

}
